package code;

import java.net.URL;
import java.util.HashMap;
import java.util.ResourceBundle;
import javafx.fxml.Initializable;
import javafx.animation.KeyFrame;
import javafx.animation.KeyValue;
import javafx.animation.Timeline;
import javafx.beans.property.DoubleProperty;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.fxml.FXMLLoader;
import javafx.fxml.FXML;
import javafx.scene.Node;
import javafx.scene.Parent;
import javafx.scene.layout.StackPane;
import javafx.scene.layout.AnchorPane;
import javafx.util.Duration;

public 
class DashController implements Initializable { //extends StackPane {

  @FXML private 
  StackPane displayPane;
  
  @FXML private 
  AnchorPane navPane;
  
  private 
  HashMap<String, Node> screens = new HashMap<>();

  @FXML private 
  void welcomeButtonOnAction(ActionEvent e) {
    this.setScreen("Welcome");
  }

  @FXML private 
  void pieChartButtonOnAction(ActionEvent e) {
    this.setScreen("PieChart");
  }

  @FXML private 
  void barChartButtonOnAction(ActionEvent e) {
    this.setScreen("BarChart");
  }

  public 
  void addScreen(String name, Node screen) { 
    screens.put(name, screen); 
  }

  public 
  boolean loadScreen(String name, String resource) {
    try { 
      FXMLLoader myLoader = new FXMLLoader(getClass().getResource(resource));
      Parent loadScreen = (Parent) myLoader.load(); 
      StackableScene myScreenControler = ((StackableScene) myLoader.getController());
      myScreenControler.setScreenParent(this); 
      addScreen(name, loadScreen); 
      return true; 
    } catch(Exception e) { 
      System.out.println(e.getMessage()); 
      return false; 
    } 
  } 

  public 
  boolean setScreen(final String name) { 

    if(screens.get(name) != null) { //screen loaded 
      final DoubleProperty opacity = this.displayPane.opacityProperty(); 
      
      final DashController dashController = this;

      //Is there is more than one screen 
      if(!this.displayPane.getChildren().isEmpty()){ 
        Timeline fade = new Timeline( 
          new KeyFrame(Duration.ZERO, new KeyValue(opacity,1.0)), 
          new KeyFrame(
            new Duration(1000), 
            new EventHandler<ActionEvent>() {
              @Override public 
              void handle(ActionEvent t) { 
                dashController.displayPane.getChildren().remove(0); 
                dashController.displayPane.getChildren().add(0, screens.get(name)); 
                Timeline fadeIn = new Timeline( 
                    new KeyFrame(Duration.ZERO, new KeyValue(opacity, 0.0)), 
                    new KeyFrame(new Duration(800), 
                    new KeyValue(opacity, 1.0))
                ); 
                fadeIn.play(); 
              } 
            }, 
            new KeyValue(opacity, 0.0))
        ); // KeyFrame.new 
        fade.play(); 
      } else { 
        //no one else been displayed, then just show 
        dashController.displayPane.setOpacity(0.0); 
        dashController.displayPane.getChildren().add(screens.get(name)); 
        Timeline fadeIn = new Timeline( 
          new KeyFrame(Duration.ZERO, new KeyValue(opacity, 0.0)), 
          new KeyFrame(new Duration(2500), new KeyValue(opacity, 1.0))
        ); 
        fadeIn.play(); 
      } 
      return true; 
    } else { 
      System.out.println("screen hasn't been loaded!\n");
      return false; 
    }
  }

  public 
  boolean unloadScreen(String name) { 
    if(screens.remove(name) == null) { 
        System.out.println("Screen didn't exist"); 
        return false; 
    } else { 
        return true; 
    } 
  } 

  @Override public 
  void initialize(URL url, ResourceBundle rb) { /*nop*/ }
}
